goal :- lots.

lots:-
   show_reversal,
   eg_count(COUNT), bench(COUNT),  fail.
lots:- pause.

eg_count(32).
eg_count(64).
eg_count(128).
eg_count(256).

show_reversal:-
   write("Reversing: "), nl,data(LIST),
     naive_reverse(LIST,RL), write(LIST), nl,
		write("Result is: "), write(RL), nl, nl.

bench(COUNT):-
   cpu_time(T0),  do_dummy(COUNT),  cpu_time(T1),
   do_bench(COUNT),  cpu_time(T2),  report(COUNT,T0,T1,T2).

do_dummy(COUNT):-
   data(LIST),	repeat(COUNT),	dummy(LIST,X),
   fail.
do_dummy(X).

dummy(X,Y).

do_bench(COUNT):-
   data(LIST),	repeat(COUNT),	do_naive_reverse(LIST,X),
   fail.
do_bench(X).

repeat(N).
repeat(N):-
   1 < N,  N1 is N - 1,  repeat(N1).

report(COUNT,T0,T1,T2):-
   TIME1 is T1 - T0,  TIME2 is T2 - T1,  TIME is TIME2 - TIME1,
   calculate_lips(COUNT,TIME,LIPS,UNITS),  nl,
   write_inside([LIPS,"lips for",COUNT,"iterations taking",TIME,UNITS,
   " (",TIME2,-,TIME1,")"]),  nl.

calculate_lips(COUNT,TIME,LIPS,sec):-
   logical_inferences(LI),     TIME100 is (TIME * 1000 + 50)/100,
   LIPS is (LI * COUNT * 10) / TIME100.

logical_inferences(LI):-
   LI is 31.0 * 32.0 / 2 .

naive_reverse([A | X],Z):-
   naive_reverse(X,Y),	append(Y,[A],Z).
naive_reverse([],[]).

append([A | X],Y,[A | Z]):-
   append(X,Y,Z).
append([],L,L).

data([1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
	21, 22, 23, 24, 25, 26, 27, 28, 29, 30]).

do_naive_reverse(L,RL):-
   naive_reverse(L,RL),  !.
